package grm;

import grm.ExecutionManagerImpl.AppInfo;
import interfaces.Grm;
import java.util.Map;
import org.omg.CORBA.ORB;
import subtypes.AsctRequestId;

public class BspFinishedExecutionManager extends StandardFinishedExecutionManager {

    //--------------------------------------------------------------------------------------
    
    BspFinishedExecutionManager(Map<String, AppInfo> currentExecutions, ORB orb, Grm grm) {
        super(currentExecutions, orb, grm);
    }

    //--------------------------------------------------------------------------------------
    
    int processRestart(AsctRequestId asctRequestId, AppInfo appInfo) {
        appInfo.bspCoord.restartBspApplication(asctRequestId, appInfo, true);
        return 0;
    }

    //--------------------------------------------------------------------------------------
    
    public int treatFinishedExecution(AppInfo appInfo, AsctRequestId asctRequestId, int restartId, int status) {

        /** Only one process from an execution shall pass each time */        
        appInfo.isActive.lock();
        try {
            
            /** This is a failure of the next iteration.
             *  Must wait until all processes from the previous iteration start. */        
            if (restartId == appInfo.currentRestart + 1) {
                try {appInfo.nextRestartCond.await();}
                catch (Exception e) {e.printStackTrace();}
            }
           
            /** Process that finished during a restart process */
            if (appInfo.isRestarting == true) {
                appInfo.bspCoord.restartBspApplication(asctRequestId, appInfo, false);
            }
            /** Process that finished due to a error during the abort process */
            else if (appInfo.isAborting == true) {
                appInfo.nodeAppIds.remove(asctRequestId.appNodeRequestId);
            }
            /** Process finished */
            else {                
                if (status == 0)     
                    processNormalExit(asctRequestId, appInfo);
                else if (status < 0) { /** TODO: Insert syncronization here */
                    appInfo.isAborting = true;
                    processAbnormalExit(asctRequestId, appInfo);
                }
                else if (status > 0) {
                    appInfo.isRestarting = true;
                    appInfo.bspCoord.restartBspApplication(asctRequestId, appInfo, true);
                }
            }            
                        
            return 0;
        }
        finally{ appInfo.isActive.unlock(); }                
    }

}
