#include <cassert>

extern "C" {
#include <lua.h>
#include <lualib.h>
#include <lauxlib.h>
}

#include "utils/c++/LuaUtils.hpp"
#include "utils/c++/OrbUtils.hpp"
#include "utils/c++/StringUtils.hpp"

#include "AsctStub.hpp"

#include <iostream>
using std::cerr;
using std::endl; 


  //----------------------------------------------------------------------
  AsctStub::AsctStub(lua_State * aState,
                     const string & aAsctIor):state(aState) {
                              
    //FIXME: This assumes that state ALREADY loaded the IDL for the GRM. Due to
    //O2's current design it is not possible to load the GRM IDL separately.
    OrbUtils::instantiateProxy (state, aAsctIor, "interfaces::Asct", "asctProxy");
    asctIor_ = aAsctIor;
  }


  //--------------------------------------------------------------------------------
  void AsctStub::acceptedExecutionRequest(const string & lrmIor,
                                          const string & lrmRequestId,
                                          const string & appMainRequestId,
                                          const string & appNodeRequestId){

    int stackTop = lua_gettop(state);
    lua_getglobal(state, "asctProxy");
    lua_pushstring(state, "acceptedExecutionRequest");
    lua_gettable(state, -2);
    lua_getglobal(state, "asctProxy");
    lua_newtable(state);//our OfferSpecs
    LuaUtils::setFieldOnTable(state, "lrmIor", lrmIor.c_str(), -1);
    LuaUtils::setFieldOnTable(state, "lrmRequestId", lrmRequestId.c_str(), -1);
    lua_pushstring(state, "asctRequestId");
    lua_newtable(state);//our AsctRequestId
    LuaUtils::setFieldOnTable(state, "appMainRequestId", appMainRequestId.c_str(), -1);
    LuaUtils::setFieldOnTable(state, "appNodeRequestId", appNodeRequestId.c_str(), -1);
    lua_settable(state, -3);
    if (lua_pcall(state, 2, 0, 0) != 0){
      cerr << "[ERROR] AsctStub::acceptedExecutionRequest->Lua error: "
           << lua_tostring(state, -1) << endl;
      lua_pop(state, 1);
    }

    lua_pop(state, 1);//remove asctProxy
    assert(stackTop == lua_gettop(state));
  }

  //--------------------------------------------------------------------------------
  void AsctStub::requestInputFiles(const string & appMainRequestId,
                                   const string & appNodeRequestId,
                                   const string & destinationPath){

    int stackTop = lua_gettop(state);
    lua_getglobal(state, "asctProxy");
    lua_pushstring(state, "requestInputFiles");
    lua_gettable(state, -2);
    lua_getglobal(state, "asctProxy");
    lua_newtable(state);
    LuaUtils::setFieldOnTable(state, "appMainRequestId", appMainRequestId.c_str(), -1);
    LuaUtils::setFieldOnTable(state, "appNodeRequestId", appNodeRequestId.c_str(), -1);
    if (lua_pcall(state, 2, 1, 0) != 0){
        cerr << "[ERROR] AsctStub::requestInputFiles->Lua error: "
             << lua_tostring(state, -1) << endl;
        lua_pop(state, 1);
    }
    unsigned int fileCount = LuaUtils::getIntFromTable(state, "n");

    for(unsigned int i = 1; i <= fileCount; i++){
      lua_rawgeti(state, -1, i);
      string filename(LuaUtils::getStringFromTable(state, "filename"));
      string filePath(destinationPath + "/" + filename);
      lua_pushstring(state, "file");
      lua_gettable(state, -2);
      LuaUtils::writeFile(state, filePath.c_str(), -1);
      lua_pop(state, 2); //removes FileStruct & File contents
    }

    lua_pop(state, 2); //removes proxy & filesequence
    assert(stackTop == lua_gettop(state));
  }


  //--------------------------------------------------------------------------------
  void AsctStub::appFinished(const string & appMainRequestId,
                             const string & appNodeRequestId){

    int stackTop = lua_gettop(state);
    lua_getglobal(state, "asctProxy");
    lua_pushstring(state, "appFinished");
    lua_gettable(state, -2);
    lua_getglobal(state, "asctProxy");
    lua_newtable(state);
    LuaUtils::setFieldOnTable(state, "appMainRequestId", appMainRequestId.c_str(), -1);
    LuaUtils::setFieldOnTable(state, "appNodeRequestId", appNodeRequestId.c_str(), -1);
    if (lua_pcall(state, 2, 0, 0) != 0){
      cerr << "[ERROR] AsctStub::appFinished->Lua error: "
           << lua_tostring(state, -1) << endl;
      lua_pop(state, 1);
    }

    lua_pop(state, 1);//removing asctProxy
    assert(stackTop == lua_gettop(state));
  }

