#ifndef ExecutionSpecs_HPP
#define ExecutionSpecs_HPP

#include <string>
#include <vector>

#include "utils/c++/StringTokenizer.hpp"


using namespace std;

 /**
   ExecutionSpecs - Holds information about an execution request

   @author Andrei Goldchleger
 */

 class ExecutionSpecs{

  private:

    string requestingAsctIor_;    /**< IOR of the ASCT requesting this execution */
    string appMainRequestId_;     /**< Main id of this execution */
    string appNodeRequestId_;     /**< Node id of this execution */
    string asctAppMainRequestId_; /**< Main id of this execution as atributed by the ASCT */
    string grmIor_;               /**< IOR of the GRM which this request will be sent  */
    string deniedExecution_;      /**< List of LRMs that denied execution of this application */
    string appId_;                /**< Identifies which application ide should be loaded from
                                    the ApplicationRepository
                               */
    string appArgs_;           /**< Arguments that will be passed to the application */
    string appConstraints_;    /**< Constraints regarding execution (in TCL) */
    string appPreferences_;    /**< Preferences regarding execution (in TCL) */
    vector<string> inputFiles_;      /**< List of files needed by this request*/
    vector<string> outputFiles_;      /**< List of files needed by this request*/


  public:


    //Named parameter idiom---------------------------------------------------

    ExecutionSpecs & requestingAsctIor(const string & aRequestingAsctIor){
      requestingAsctIor_= aRequestingAsctIor;
      return (*this);
    }

    ExecutionSpecs & appMainRequestId(const string & appMainRequestId){
      appMainRequestId_= appMainRequestId;
      return (*this);
    }

    ExecutionSpecs & asctAppMainRequestId(const string & asctAppMainRequestId){
      asctAppMainRequestId_= asctAppMainRequestId;
      return (*this);
    }

    ExecutionSpecs & appNodeRequestId(const string & appNodeRequestId){
      appNodeRequestId_= appNodeRequestId;
      return (*this);
    }

    ExecutionSpecs & grmIor(const string & aGrmIor){
      grmIor_= aGrmIor;
      return (*this);
    }

    ExecutionSpecs & deniedExecution(const string & aDeniedExecution){
      deniedExecution_= aDeniedExecution;
      return (*this);
    }

    ExecutionSpecs & appId(const string & aAppId){
      appId_= aAppId;
      return (*this);
    }

    ExecutionSpecs & appArgs(const string & aAppArgs){
      appArgs_= aAppArgs;
      return (*this);
    }

    ExecutionSpecs & appConstraints(const string & aAppConstraints){
      appConstraints_= aAppConstraints;
      return (*this);
    }

    ExecutionSpecs & appPreferences(const string & aAppPreferences){
      appPreferences_= aAppPreferences;
      return (*this);
    }

    //This method assumes that filenames are separated by whitespace.
    ExecutionSpecs & inputFiles(const string & aInputFiles){
      StringTokenizer st(aInputFiles);
      while(st.hasMoreTokens())
       inputFiles_.push_back(st.nextToken());
      return (*this);
    }

    ExecutionSpecs & inputFiles(vector<string> aInputFiles){
      inputFiles_ = aInputFiles;
      return (*this);
    }

    ExecutionSpecs & outputFiles(vector<string> aOutputFiles){
      outputFiles_ = aOutputFiles;
      return (*this);
    }


   //getters-------------------------------------------------------------
   const string & requestingAsctIor() const{return requestingAsctIor_;}
   const string & appMainRequestId() const{return appMainRequestId_;}
   const string & asctAppMainRequestId() const{return asctAppMainRequestId_;}
   const string & appNodeRequestId() const{return appNodeRequestId_;}
   const string & grmIor() const{return  grmIor_;}
   const string & deniedExecution() const{return  deniedExecution_;}
   const string & appId() const{return   appId_;}
   const string & appArgs() const{return   appArgs_;}
   const string & appConstraints() const{return  appConstraints_;}
   const string & appPreferences() const{return  appPreferences_;}
   const vector<string> & inputFiles() const{return inputFiles_;}
   const vector<string> & outputFiles() const{return outputFiles_;}


 };


#endif//ExecutionSpecs_HPP

