#include <cassert>

#include "LocStaticInfo.hpp"
#include "LocDynamicInfo.hpp"
#include "ExecutionSpecs.hpp"

extern "C" {
#include <lua.h>
#include <lualib.h>
#include <lauxlib.h>
}


#include "utils/c++/LuaUtils.hpp"
#include "utils/c++/OrbUtils.hpp"

#include "GrmStub.hpp"

#include <iostream>


  //----------------------------------------------------------------------
  GrmStub::GrmStub(lua_State * aState, const string & aGrmIor)
     : state(aState), grmIor(aGrmIor) {

    //FIXME: This assumes that state ALREADY loaded the IDL for the GRM. Due to
    //O2's current design it is not possible to load the GRM IDL separately.
    OrbUtils::instantiateProxy (state, aGrmIor, "IDL:interfaces/Grm:1.0", "grmProxy");
  }


  //----------------------------------------------------------------------
  void GrmStub::registerLrm(const string & lrmIor, const LocStaticInfo  & lsi){

    int stackTop = lua_gettop(state);
    lua_getglobal(state, "grmProxy");
    lua_pushstring(state, "registerLrm");
    lua_gettable(state, -2);
    lua_getglobal(state, "grmProxy");
    lua_pushstring(state, lrmIor.c_str());
    //Creates a table with static info
    lua_newtable(state);
    LuaUtils::setFieldOnTable(state,"hostName", lsi.getHostName(), -1);
    LuaUtils::setFieldOnTable(state, "osName", lsi.getOsName(), -1);
    LuaUtils::setFieldOnTable(state, "osVersion", lsi.getOsVersion(), -1);
    LuaUtils::setFieldOnTable(state, "processorName", lsi.getProcessorName(), -1);
    LuaUtils::setFieldOnTable(state, "processorMhz", lsi.getProcessorMhz(), -1);
    LuaUtils::setFieldOnTable(state, "totalRAM", lsi.getTotalRAM(), -1);
    LuaUtils::setFieldOnTable(state, "totalSwap", lsi.getTotalSwap(), -1);
    if (lua_pcall(state, 3, 0, 0) != 0){
      cerr << "[ERROR] GrmStub::registerLrm->Lua error: "
           << lua_tostring(state, -1) << endl;
      lua_pop(state, 1);
    }

    lua_pop(state,1);//Removes grmProxy from stack
    assert(stackTop == lua_gettop(state));

  }

  //----------------------------------------------------------------------
  void GrmStub::sendUpdate(const string & lrmIor, const LocDynamicInfo  & ldi){

    int stackTop = lua_gettop(state);
    lua_getglobal(state, "grmProxy");
    lua_pushstring(state, "sendUpdate");
    lua_gettable(state, -2);
    lua_getglobal(state, "grmProxy");
    lua_pushstring(state, lrmIor.c_str());
    //Creates a table with dynamic info
    lua_newtable(state);
    LuaUtils::setFieldOnTable(state, "freeRAM", ldi.getFreeRAM(), -1);
    LuaUtils::setFieldOnTable(state, "freeSwap", ldi.getFreeSwap(), -1);
    LuaUtils::setFieldOnTable(state, "fsFree", ldi.getFsFree(), -1);
    LuaUtils::setFieldOnTable(state, "filesFree", ldi.getFilesFree(), -1);
    LuaUtils::setFieldOnTable(state, "cpuUsage", ldi.getCpuUsage(), -1);
    LuaUtils::setFieldOnTable(state, "fsTotal", ldi.getFsTotal(), -1);
    LuaUtils::setFieldOnTable(state, "filesTotal", ldi.getFilesTotal(), -1);

    if (lua_pcall(state, 3, 0, 0) != 0){
      cerr << "[ERROR] GrmStub::sendUpdate->Lua error: "
           << lua_tostring(state, -1) << endl;
      lua_pop(state, 1);
    }

    lua_pop(state,1);//Removes grmProxy from stack
    assert(stackTop == lua_gettop(state));

  }
