package asctGui;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import java.awt.BorderLayout;
import java.awt.Dimension;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.JTree;

import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;

import javax.swing.JSplitPane;

import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import java.nio.channels.FileChannel;

import java.awt.Container;//for getRootFrame

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

  class FileViewPanel extends JPanel{

    private DefaultMutableTreeNode rootNode;
    private final JTree treeView;
    private FileDisplayPanel displayPanel;
    
    //-------------------------------------------------------------------------
    public FileViewPanel(String path){

      displayPanel = new FileDisplayPanel();
      rootNode = createTree(new File(path));
      treeView = new JTree(rootNode);
      treeView.getSelectionModel().
       setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
      treeView.setRootVisible(false);

      treeView.addTreeSelectionListener(
        new TreeSelectionListener(){
          public void valueChanged(TreeSelectionEvent e) {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                           treeView.getLastSelectedPathComponent();
            if(node == null)
              return;
            if(node.isLeaf())
             if(node.getUserObject().getClass() == NamePathHolder.class)
              displayPanel.loadText(((NamePathHolder) node.getUserObject()).path());
          }
        });

      treeView.addMouseListener (new TreeMouseAdapter(treeView));

      JScrollPane treeScroll = new JScrollPane(treeView);
      treeScroll.setPreferredSize(new Dimension(200,0));

      JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
      splitPane.setLeftComponent(treeScroll);
      splitPane.setRightComponent(displayPanel);

      this.setLayout(new BorderLayout());
      this.add(splitPane, BorderLayout.CENTER);

    }

    //-------------------------------------------------------------------------
    private DefaultMutableTreeNode createTree(File rootFile){

      //Base
      if(rootFile.isFile()){
        return new DefaultMutableTreeNode(
                 new NamePathHolder(rootFile.getName(), rootFile.getPath()));
      }
      //Recursion
      else{
        File [] rootContents = rootFile.listFiles();
        DefaultMutableTreeNode rootNode  = null;
        if(rootContents.length == 0)
          rootNode =
           new DefaultMutableTreeNode("Node "
                                      + (Integer.parseInt(rootFile.getName()) + 1)
                                      + ": No Output Files");
        else
          rootNode =
           new DefaultMutableTreeNode("Node " + (Integer.parseInt(rootFile.getName()) + 1));
        for(int i = 0; i < rootContents.length; i++)
          if(rootContents[i].isFile() || rootContents[i].isDirectory())
            rootNode.add(createTree(rootContents[i]));
          return rootNode;
      }

    }
  }//class

  //=========================================================================
  class NamePathHolder{

    private String name_;
    private String path_;

    NamePathHolder(String name, String path){
      name_ = name;
      path_ = path;
    }

    public String path(){ return path_; }
    public String toString(){ return name_; }

  }//class

  //=========================================================================
  class TreeMouseAdapter extends MouseAdapter{

    private JTree treeView;

    TreeMouseAdapter(JTree aTreeView){ treeView = aTreeView; }

    //-----------------------------------------------------------------------
    public void mousePressed (MouseEvent e) {
      processMouseEvent(e);
    }

    //-----------------------------------------------------------------------
    public void mouseReleased (MouseEvent e) {
        processMouseEvent(e);

      }

    //-----------------------------------------------------------------------
    private void processMouseEvent(MouseEvent e){
       if (e.isPopupTrigger())
       if(e.getComponent().getClass() == JTree.class){
           final TreePath selPath = treeView.getPathForLocation(e.getX(), e.getY());
           if(selPath != null)
             if(selPath.getLastPathComponent().getClass() == DefaultMutableTreeNode.class)
               if(((DefaultMutableTreeNode)selPath.getLastPathComponent()).isLeaf()){
                 JPopupMenu popup = new JPopupMenu ();
                 JMenuItem saveItem = new JMenuItem ("Save");
                 JMenuItem deleteItem = new JMenuItem ("Delete");
                 saveItem.addActionListener(new ActionListener(){
                   public void actionPerformed(ActionEvent e){
                     DefaultMutableTreeNode selectedNode =
                       (DefaultMutableTreeNode) selPath.getLastPathComponent();
                      saveFile(((NamePathHolder)selectedNode.getUserObject()).path());
                    }
                 });
                 deleteItem.addActionListener(new ActionListener(){
                   public void actionPerformed(ActionEvent e){
                      DefaultMutableTreeNode selectedNode =
                       (DefaultMutableTreeNode) selPath.getLastPathComponent();
                      deleteFile(selectedNode);
                    }
                 });
                 popup.add (saveItem);
                 popup.add (deleteItem);
                 popup.setInvoker (e.getComponent());
                 popup.show (e.getComponent(), e.getX(), e.getY());
               }
        }
    }//Method


    //-----------------------------------------------------------------------
    private void saveFile(String srcPath){
      JFileChooser fileChooser = new JFileChooser (".");
      fileChooser.setDialogTitle("Save File");
      fileChooser.setApproveButtonText("Save");
      if (fileChooser.showOpenDialog(treeView.getTopLevelAncestor()) == JFileChooser.APPROVE_OPTION){
        try {
          FileChannel srcChannel = new FileInputStream(srcPath).getChannel();
          FileChannel dstChannel = new FileOutputStream(fileChooser.getSelectedFile()).getChannel();
          dstChannel.transferFrom(srcChannel, 0, srcChannel.size());
          srcChannel.close();
          dstChannel.close();
        }
        catch (IOException e) {
          System.out.println("FileTreePanel::Error Saving file");
          e.printStackTrace();
        }
      }
    }

    //-----------------------------------------------------------------------
    private void deleteFile(DefaultMutableTreeNode delNode){
      Object [] options = new Object [] {"Delete",  "Cancel"};
      int selection = JOptionPane.showOptionDialog(treeView.getTopLevelAncestor(),
                                   "Do you really want to delete the file: \n" +
                                   ((NamePathHolder)delNode.getUserObject()).toString() + " ?\n",
                                   "Confirm Deletion",
                                   JOptionPane.YES_NO_OPTION,
                                   JOptionPane.WARNING_MESSAGE,
                                   null,//default icon
                                   options,
                                   options[1]);
      if(selection == JOptionPane.YES_OPTION){
        File delFile = new File(((NamePathHolder)delNode.getUserObject()).path());
        delFile.delete();
        if(delNode.getParent().getChildCount() == 1)
          ((DefaultTreeModel)treeView.getModel())
              .removeNodeFromParent((DefaultMutableTreeNode) delNode.getParent());
        ((DefaultTreeModel)treeView.getModel()).removeNodeFromParent(delNode);
        //FIXME: NEED to recurse if depth > 2
      }
    }//method

  }//class


